@@ -3,6 +3,8 @@
 #ifndef ACLK_SCHEMA_WRAPPER_CONNECTION_H
 #define ACLK_SCHEMA_WRAPPER_CONNECTION_H
 
+#include "capability.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -15,6 +17,8 @@ typedef struct {
 
     unsigned int lwt:1;
 
+    struct capability *capabilities;
+
 // TODO in future optional fields
 // > 15 optional fields:
 // How long the system was running until connection (only applicable when reachable=true)
