@@ -4,7 +4,9 @@
 #define ACLK_SCHEMA_WRAPPER_NODE_INFO_H
 
 #include <stdlib.h>
+#include <stdint.h>
 
+#include "capability.h"
 #include "database/rrd.h"
 
 #ifdef __cplusplus
@@ -48,12 +50,9 @@ struct aclk_node_info {
 
     char *custom_info;
 
-    char **services;
-    size_t service_count;
-
     char *machine_guid;
 
-    struct label *host_labels_head;
+    DICTIONARY *host_labels_ptr;
 
     struct machine_learning_info ml_info;
 };
@@ -67,10 +66,26 @@ struct update_node_info {
     int child;
 
     struct machine_learning_info ml_info;
+
+    struct capability *node_capabilities;
+    struct capability *node_instance_capabilities;
+};
+
+struct collector_info {
+    char *module;
+    char *plugin;
+};
+
+struct update_node_collectors {
+    char *claim_id;
+    char *node_id;
+    DICTIONARY *node_collectors;
 };
 
 char *generate_update_node_info_message(size_t *len, struct update_node_info *info);
 
+char *generate_update_node_collectors_message(size_t *len, struct update_node_collectors *collectors);
+
 #ifdef __cplusplus
 }
 #endif
