@@ -13,5 +13,8 @@
 #include "alarm_config.h"
 #include "alarm_stream.h"
 #include "node_info.h"
+#include "capability.h"
+#include "context_stream.h"
+#include "context.h"
 
 #endif /* SCHEMA_WRAPPERS_H */
