@@ -31,7 +31,7 @@ static char *claiming_errors[] = {
 /* Retrieve the claim id for the agent.
  * Caller owns the string.
 */
-char *is_agent_claimed()
+char *get_agent_claimid()
 {
     char *result;
     rrdhost_aclk_state_lock(localhost);
@@ -134,6 +134,10 @@ void load_claiming_state(void)
     netdata_cloud_setting = 0;
 #else
     uuid_t uuid;
+
+    // Propagate into aclk and registry. Be kind of atomic...
+    appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", DEFAULT_CLOUD_BASE_URL);
+
     rrdhost_aclk_state_lock(localhost);
     if (localhost->aclk_state.claimed_id) {
         if (aclk_connected)
@@ -148,9 +152,6 @@ void load_claiming_state(void)
     }
     aclk_disable_runtime = 0;
 
-    // Propagate into aclk and registry. Be kind of atomic...
-    appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", DEFAULT_CLOUD_BASE_URL);
-
     char filename[FILENAME_MAX + 1];
     snprintfz(filename, FILENAME_MAX, "%s/cloud.d/claimed_id", netdata_configured_varlib_dir);
 
@@ -161,7 +162,11 @@ void load_claiming_state(void)
         freez(claimed_id);
         claimed_id = NULL;
     }
-    localhost->aclk_state.claimed_id = claimed_id;
+
+    if(claimed_id) {
+        localhost->aclk_state.claimed_id = mallocz(UUID_STR_LEN);
+        uuid_unparse_lower(uuid, localhost->aclk_state.claimed_id);
+    }
 
     invalidate_node_instances(&localhost->host_uuid, claimed_id ? &uuid : NULL);
     store_claim_id(&localhost->host_uuid, claimed_id ? &uuid : NULL);
@@ -172,6 +177,8 @@ void load_claiming_state(void)
         return;
     }
 
+    freez(claimed_id);
+
     info("File '%s' was found. Setting state to AGENT_CLAIMED.", filename);
     netdata_cloud_setting = appconfig_get_boolean(&cloud_config, CONFIG_SECTION_GLOBAL, "enabled", 1);
 #endif
