@@ -379,6 +379,10 @@ typedef struct ebpf_bandwidth {
     uint64_t retransmit;         // Number of times tcp_retransmit was called
     uint64_t call_udp_sent;      // Number of times udp_sendmsg was called
     uint64_t call_udp_received;  // Number of times udp_recvmsg was called
+    uint64_t close;              // Number of times tcp_close was called
+    uint64_t drop;               // THIS IS NOT USED FOR WHILE, we are in groom section
+    uint32_t tcp_v4_connection;  // Number of times tcp_v4_connection was called.
+    uint32_t tcp_v6_connection;  // Number of times tcp_v6_connection was called.
 } ebpf_bandwidth_t;
 
 /**
@@ -429,8 +433,6 @@ extern size_t read_bandwidth_statistic_using_pid_on_target(ebpf_bandwidth_t **ep
 
 extern void collect_data_for_all_processes(int tbl_pid_stats_fd);
 
-extern void clean_global_memory();
-
 extern ebpf_process_stat_t **global_process_stats;
 extern ebpf_process_publish_apps_t **current_apps_data;
 extern netdata_publish_cachestat_t **cachestat_pid;
