@@ -8,8 +8,6 @@ static char *sync_counter_dimension_name[NETDATA_SYNC_IDX_END] = { "sync", "sync
 static netdata_syscall_stat_t sync_counter_aggregated_data[NETDATA_SYNC_IDX_END];
 static netdata_publish_syscall_t sync_counter_publish_aggregated[NETDATA_SYNC_IDX_END];
 
-static int read_thread_closed = 1;
-
 static netdata_idx_t sync_hash_values[NETDATA_SYNC_IDX_END];
 
 struct netdata_static_thread sync_threads = {"SYNC KERNEL", NULL, NULL, 1,
@@ -43,15 +41,211 @@ struct config sync_config = { .first_section = NULL,
     .index = { .avl_tree = { .root = NULL, .compar = appconfig_section_compare },
         .rwlock = AVL_LOCK_INITIALIZER } };
 
-ebpf_sync_syscalls_t local_syscalls[] = {
-    {.syscall = "sync", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = "syncfs", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = "msync", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = "fsync", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = "fdatasync", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = "sync_file_range", .enabled = CONFIG_BOOLEAN_YES, .objects = NULL, .probe_links = NULL},
-    {.syscall = NULL, .enabled = CONFIG_BOOLEAN_NO, .objects = NULL, .probe_links = NULL}
-};
+netdata_ebpf_targets_t sync_targets[] = { {.name = NETDATA_SYSCALLS_SYNC, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NETDATA_SYSCALLS_SYNCFS, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NETDATA_SYSCALLS_MSYNC, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NETDATA_SYSCALLS_FSYNC, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NETDATA_SYSCALLS_FDATASYNC, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NETDATA_SYSCALLS_SYNC_FILE_RANGE, .mode = EBPF_LOAD_TRAMPOLINE},
+                                          {.name = NULL, .mode = EBPF_LOAD_TRAMPOLINE}};
+static enum ebpf_threads_status ebpf_sync_exited = NETDATA_THREAD_EBPF_RUNNING;
+
+
+#ifdef LIBBPF_MAJOR_VERSION
+/*****************************************************************
+ *
+ *  BTF FUNCTIONS
+ *
+ *****************************************************************/
+
+/**
+ * Disable probe
+ *
+ * Disable kprobe to use another method.
+ *
+ * @param obj is the main structure for bpf objects.
+ */
+static inline void ebpf_sync_disable_probe(struct sync_bpf *obj)
+{
+    bpf_program__set_autoload(obj->progs.netdata_sync_kprobe, false);
+}
+
+/**
+ * Disable tramppoline
+ *
+ * Disable trampoline to use another method.
+ *
+ * @param obj is the main structure for bpf objects.
+ */
+static inline void ebpf_sync_disable_trampoline(struct sync_bpf *obj)
+{
+    bpf_program__set_autoload(obj->progs.netdata_sync_fentry, false);
+}
+
+/**
+ * Disable tracepoint
+ *
+ * Disable tracepoints according information given.
+ *
+ * @param obj  object loaded
+ * @param idx  Which syscall will not be disabled
+ */
+void ebpf_sync_disable_tracepoints(struct sync_bpf *obj, sync_syscalls_index_t idx)
+{
+    if (idx != NETDATA_SYNC_SYNC_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_sync_entry, false);
+
+    if (idx != NETDATA_SYNC_SYNCFS_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_syncfs_entry, false);
+
+    if (idx != NETDATA_SYNC_MSYNC_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_msync_entry, false);
+
+    if (idx != NETDATA_SYNC_FSYNC_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_fsync_entry, false);
+
+    if (idx != NETDATA_SYNC_FDATASYNC_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_fdatasync_entry, false);
+
+    if (idx != NETDATA_SYNC_SYNC_FILE_RANGE_IDX)
+        bpf_program__set_autoload(obj->progs.netdata_sync_file_range_entry, false);
+}
+
+/**
+ * Set hash tables
+ *
+ * Set the values for maps according the value given by kernel.
+ *
+ * @param obj is the main structure for bpf objects.
+ * @param idx    the index for the main structure
+ */
+static void ebpf_sync_set_hash_tables(struct sync_bpf *obj, sync_syscalls_index_t idx)
+{
+    sync_maps[idx].map_fd = bpf_map__fd(obj->maps.tbl_sync);
+}
+
+/**
+ * Load and attach
+ *
+ * Load and attach the eBPF code in kernel.
+ *
+ * @param obj    is the main structure for bpf objects.
+ * @param em     the structure with configuration
+ * @param target the syscall that we are attaching a tracer.
+ * @param idx    the index for the main structure
+ *
+ * @return it returns 0 on succes and -1 otherwise
+ */
+static inline int ebpf_sync_load_and_attach(struct sync_bpf *obj, ebpf_module_t *em, char *target,
+                                            sync_syscalls_index_t idx)
+{
+    netdata_ebpf_targets_t *synct = em->targets;
+    netdata_ebpf_program_loaded_t test = synct[NETDATA_SYNC_SYNC_IDX].mode;
+
+    if (test == EBPF_LOAD_TRAMPOLINE) {
+        ebpf_sync_disable_probe(obj);
+        ebpf_sync_disable_tracepoints(obj, NETDATA_SYNC_IDX_END);
+
+        bpf_program__set_attach_target(obj->progs.netdata_sync_fentry, 0,
+                                       target);
+    } else if (test == EBPF_LOAD_PROBE ||
+    test == EBPF_LOAD_RETPROBE) {
+        ebpf_sync_disable_tracepoints(obj, NETDATA_SYNC_IDX_END);
+        ebpf_sync_disable_trampoline(obj);
+    } else {
+        ebpf_sync_disable_probe(obj);
+        ebpf_sync_disable_trampoline(obj);
+
+        ebpf_sync_disable_tracepoints(obj, idx);
+    }
+
+    int ret = sync_bpf__load(obj);
+    if (!ret) {
+        if (test != EBPF_LOAD_PROBE && test != EBPF_LOAD_RETPROBE) {
+            ret = sync_bpf__attach(obj);
+        } else {
+            obj->links.netdata_sync_kprobe = bpf_program__attach_kprobe(obj->progs.netdata_sync_kprobe,
+                                                                        false, target);
+            ret = (int)libbpf_get_error(obj->links.netdata_sync_kprobe);
+        }
+
+        if (!ret)
+            ebpf_sync_set_hash_tables(obj, idx);
+    }
+
+    return ret;
+}
+#endif
+
+/*****************************************************************
+ *
+ *  CLEANUP THREAD
+ *
+ *****************************************************************/
+
+/**
+ * Cleanup Objects
+ *
+ * Cleanup loaded objects when thread was initialized.
+ */
+void ebpf_sync_cleanup_objects()
+{
+    int i;
+    for (i = 0; local_syscalls[i].syscall; i++) {
+        ebpf_sync_syscalls_t *w = &local_syscalls[i];
+        if (w->probe_links) {
+            struct bpf_program *prog;
+            size_t j = 0 ;
+            bpf_object__for_each_program(prog, w->objects) {
+                bpf_link__destroy(w->probe_links[j]);
+                j++;
+            }
+            freez(w->probe_links);
+            if (w->objects)
+                bpf_object__close(w->objects);
+        }
+#ifdef LIBBPF_MAJOR_VERSION
+        else if (w->sync_obj)
+            sync_bpf__destroy(w->sync_obj);
+#endif
+    }
+}
+
+/**
+ * Exit
+ *
+ * Clean up the main thread.
+ *
+ * @param ptr thread data.
+ */
+static void ebpf_sync_exit(void *ptr)
+{
+    ebpf_module_t *em = (ebpf_module_t *)ptr;
+    if (!em->enabled) {
+        em->enabled = NETDATA_MAIN_THREAD_EXITED;
+        return;
+    }
+
+    ebpf_sync_exited = NETDATA_THREAD_EBPF_STOPPING;
+}
+
+/**
+ * Clean up the main thread.
+ *
+ * @param ptr thread data.
+ */
+static void ebpf_sync_cleanup(void *ptr)
+{
+    ebpf_module_t *em = (ebpf_module_t *)ptr;
+    if (ebpf_sync_exited != NETDATA_THREAD_EBPF_STOPPED)
+        return;
+
+    ebpf_sync_cleanup_objects();
+    freez(sync_threads.thread);
+
+    sync_threads.enabled = NETDATA_MAIN_THREAD_EXITED;
+    em->enabled = NETDATA_MAIN_THREAD_EXITED;
+}
 
 /*****************************************************************
  *
@@ -59,6 +253,29 @@ ebpf_sync_syscalls_t local_syscalls[] = {
  *
  *****************************************************************/
 
+/**
+ * Load Legacy
+ *
+ * Load legacy code.
+ *
+ * @param w   is the sync output structure with pointers to objects loaded.
+ * @param em  is structure with configuration
+ *
+ * @return 0 on success and -1 otherwise.
+ */
+static int ebpf_sync_load_legacy(ebpf_sync_syscalls_t *w, ebpf_module_t *em)
+{
+    em->thread_name = w->syscall;
+    if (!w->probe_links) {
+        w->probe_links = ebpf_load_program(ebpf_plugin_dir, em, running_on_kernel, isrh, &w->objects);
+        if (!w->probe_links) {
+            return -1;
+        }
+    }
+
+    return 0;
+}
+
 /*
  * Initialize Syscalls
  *
@@ -70,15 +287,36 @@ static int ebpf_sync_initialize_syscall(ebpf_module_t *em)
 {
     int i;
     const char *saved_name = em->thread_name;
+    int errors = 0;
     for (i = 0; local_syscalls[i].syscall; i++) {
         ebpf_sync_syscalls_t *w = &local_syscalls[i];
-        if (!w->probe_links && w->enabled) {
-            em->thread_name = w->syscall;
-            w->probe_links = ebpf_load_program(ebpf_plugin_dir, em, running_on_kernel, isrh, &w->objects);
-            if (!w->probe_links) {
+        if (w->enabled) {
+            if (em->load == EBPF_LOAD_LEGACY) {
+                if (ebpf_sync_load_legacy(w, em))
+                    errors++;
+
                 em->thread_name = saved_name;
-                return -1;
             }
+#ifdef LIBBPF_MAJOR_VERSION
+            else {
+                char syscall[NETDATA_EBPF_MAX_SYSCALL_LENGTH];
+                ebpf_select_host_prefix(syscall, NETDATA_EBPF_MAX_SYSCALL_LENGTH, w->syscall, running_on_kernel);
+                w->sync_obj = sync_bpf__open();
+                if (!w->sync_obj) {
+                    errors++;
+                } else {
+                    if (ebpf_is_function_inside_btf(default_btf, syscall)) {
+                        if (ebpf_sync_load_and_attach(w->sync_obj, em, syscall, i)) {
+                            errors++;
+                        }
+                    } else {
+                        if (ebpf_sync_load_legacy(w, em))
+                            errors++;
+                    }
+                    em->thread_name = saved_name;
+                }
+            }
+#endif
         }
     }
     em->thread_name = saved_name;
@@ -87,7 +325,7 @@ static int ebpf_sync_initialize_syscall(ebpf_module_t *em)
     memset(sync_counter_publish_aggregated, 0 , NETDATA_SYNC_IDX_END * sizeof(netdata_publish_syscall_t));
     memset(sync_hash_values, 0 , NETDATA_SYNC_IDX_END * sizeof(netdata_idx_t));
 
-    return 0;
+    return (errors) ? -1 : 0;
 }
 
 /*****************************************************************
@@ -127,21 +365,25 @@ static void read_global_table()
  */
 void *ebpf_sync_read_hash(void *ptr)
 {
+    netdata_thread_cleanup_push(ebpf_sync_cleanup, ptr);
     ebpf_module_t *em = (ebpf_module_t *)ptr;
-    read_thread_closed = 0;
 
     heartbeat_t hb;
     heartbeat_init(&hb);
     usec_t step = NETDATA_EBPF_SYNC_SLEEP_MS * em->update_every;
 
-    while (!close_ebpf_plugin) {
+    while (ebpf_sync_exited == NETDATA_THREAD_EBPF_RUNNING) {
         usec_t dt = heartbeat_next(&hb, step);
         (void)dt;
+        if (ebpf_sync_exited == NETDATA_THREAD_EBPF_STOPPING)
+            break;
 
         read_global_table();
     }
-    read_thread_closed = 1;
 
+    ebpf_sync_exited = NETDATA_THREAD_EBPF_STOPPED;
+
+    netdata_thread_cleanup_pop(1);
     return NULL;
 }
 
@@ -207,79 +449,26 @@ static void sync_collector(ebpf_module_t *em)
     sync_threads.thread = mallocz(sizeof(netdata_thread_t));
     sync_threads.start_routine = ebpf_sync_read_hash;
 
-    netdata_thread_create(sync_threads.thread, sync_threads.name, NETDATA_THREAD_OPTION_JOINABLE,
+    netdata_thread_create(sync_threads.thread, sync_threads.name, NETDATA_THREAD_OPTION_DEFAULT,
                           ebpf_sync_read_hash, em);
 
-    int update_every = em->update_every;
-    int counter = update_every - 1;
-    while (!close_ebpf_plugin) {
-        pthread_mutex_lock(&collect_data_mutex);
-        pthread_cond_wait(&collect_data_cond_var, &collect_data_mutex);
-
-        if (++counter == update_every) {
-            counter = 0;
-            pthread_mutex_lock(&lock);
-
-            sync_send_data();
-
-            pthread_mutex_unlock(&lock);
-        }
-        pthread_mutex_unlock(&collect_data_mutex);
-    }
-}
-
-
-/*****************************************************************
- *
- *  CLEANUP THREAD
- *
- *****************************************************************/
-
-/**
- * Cleanup Objects
- *
- * Cleanup loaded objects when thread was initialized.
- */
-void ebpf_sync_cleanup_objects()
-{
-    int i;
-    for (i = 0; local_syscalls[i].syscall; i++) {
-        ebpf_sync_syscalls_t *w = &local_syscalls[i];
-        if (w->probe_links) {
-            struct bpf_program *prog;
-            size_t j = 0 ;
-            bpf_object__for_each_program(prog, w->objects) {
-                bpf_link__destroy(w->probe_links[j]);
-                j++;
-            }
-            bpf_object__close(w->objects);
-        }
-    }
-}
-
-/**
- * Clean up the main thread.
- *
- * @param ptr thread data.
- */
-static void ebpf_sync_cleanup(void *ptr)
-{
-    ebpf_module_t *em = (ebpf_module_t *)ptr;
-    if (!em->enabled)
-        return;
-
     heartbeat_t hb;
     heartbeat_init(&hb);
-    uint32_t tick = 2*USEC_PER_MS;
-    while (!read_thread_closed) {
-        usec_t dt = heartbeat_next(&hb, tick);
-        UNUSED(dt);
-    }
+    usec_t step = em->update_every * USEC_PER_SEC;
+    while (!ebpf_exit_plugin) {
+        (void)heartbeat_next(&hb, step);
+        if (ebpf_exit_plugin)
+            break;
 
-    ebpf_sync_cleanup_objects();
-    freez(sync_threads.thread);
+        pthread_mutex_lock(&lock);
+
+        sync_send_data();
+
+        pthread_mutex_unlock(&lock);
+    }
 }
 
+
 /*****************************************************************
  *
  *  MAIN THREAD
@@ -376,7 +565,7 @@ static void ebpf_sync_parse_syscalls()
  */
 void *ebpf_sync_thread(void *ptr)
 {
-    netdata_thread_cleanup_push(ebpf_sync_cleanup, ptr);
+    netdata_thread_cleanup_push(ebpf_sync_exit, ptr);
 
     ebpf_module_t *em = (ebpf_module_t *)ptr;
     em->maps = sync_maps;
@@ -386,6 +575,9 @@ void *ebpf_sync_thread(void *ptr)
     if (!em->enabled)
         goto endsync;
 
+#ifdef LIBBPF_MAJOR_VERSION
+    ebpf_adjust_thread_load(em, default_btf);
+#endif
     if (ebpf_sync_initialize_syscall(em)) {
         em->enabled = CONFIG_BOOLEAN_NO;
         goto endsync;
