@@ -201,7 +201,7 @@ static unsigned long long int bcache_read_number_with_units(const char *filename
         static int unknown_units_error = 10;
 
         char *end = NULL;
-        long double value = str2ld(buffer, &end);
+        NETDATA_DOUBLE value = str2ndd(buffer, &end);
         if(end && *end) {
             if(*end == 'k')
                 return (unsigned long long int)(value * 1024.0);
@@ -830,6 +830,30 @@ static struct disk *get_disk(unsigned long major, unsigned long minor, char *dis
     return d;
 }
 
+static void add_labels_to_disk(struct disk *d, RRDSET *st) {
+    rrdlabels_add(st->state->chart_labels, "device", d->disk, RRDLABEL_SRC_AUTO);
+    rrdlabels_add(st->state->chart_labels, "mount_point", d->mount_point, RRDLABEL_SRC_AUTO);
+
+    switch (d->type) {
+        default:
+        case DISK_TYPE_UNKNOWN:
+            rrdlabels_add(st->state->chart_labels, "device_type", "unknown", RRDLABEL_SRC_AUTO);
+            break;
+
+        case DISK_TYPE_PHYSICAL:
+            rrdlabels_add(st->state->chart_labels, "device_type", "physical", RRDLABEL_SRC_AUTO);
+            break;
+
+        case DISK_TYPE_PARTITION:
+            rrdlabels_add(st->state->chart_labels, "device_type", "partition", RRDLABEL_SRC_AUTO);
+            break;
+
+        case DISK_TYPE_VIRTUAL:
+            rrdlabels_add(st->state->chart_labels, "device_type", "virtual", RRDLABEL_SRC_AUTO);
+            break;
+    }
+}
+
 int do_proc_diskstats(int update_every, usec_t dt) {
     static procfile *ff = NULL;
 
@@ -1067,6 +1091,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 d->rd_io_reads  = rrddim_add(d->st_io, "reads",  NULL, d->sector_size, 1024,      RRD_ALGORITHM_INCREMENTAL);
                 d->rd_io_writes = rrddim_add(d->st_io, "writes", NULL, d->sector_size * -1, 1024, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_io);
             }
             else rrdset_next(d->st_io);
 
@@ -1094,8 +1120,9 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                         , RRDSET_TYPE_AREA
                 );
 
-                d->rd_io_discards =
-                    rrddim_add(d->st_ext_io, "discards", NULL, d->sector_size, 1024, RRD_ALGORITHM_INCREMENTAL);
+                d->rd_io_discards = rrddim_add(d->st_ext_io, "discards", NULL, d->sector_size, 1024, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_ext_io);
             } else
                 rrdset_next(d->st_ext_io);
 
@@ -1130,6 +1157,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 d->rd_ops_reads  = rrddim_add(d->st_ops, "reads",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_ops_writes = rrddim_add(d->st_ops, "writes", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_ops);
             }
             else rrdset_next(d->st_ops);
 
@@ -1162,7 +1191,10 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                 d->rd_ops_discards = rrddim_add(d->st_ext_ops, "discards", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                 if (do_fl_stats)
                     d->rd_ops_flushes = rrddim_add(d->st_ext_ops, "flushes", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-            } else
+
+                add_labels_to_disk(d, d->st_ext_ops);
+            }
+            else
                 rrdset_next(d->st_ext_ops);
 
             last_discards = rrddim_set_by_pointer(d->st_ext_ops, d->rd_ops_discards, discards);
@@ -1196,6 +1228,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                 rrdset_flag_set(d->st_qops, RRDSET_FLAG_DETAIL);
 
                 d->rd_qops_operations = rrddim_add(d->st_qops, "operations", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                add_labels_to_disk(d, d->st_qops);
             }
             else rrdset_next(d->st_qops);
 
@@ -1228,6 +1262,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                 rrdset_flag_set(d->st_backlog, RRDSET_FLAG_DETAIL);
 
                 d->rd_backlog_backlog = rrddim_add(d->st_backlog, "backlog", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_backlog);
             }
             else rrdset_next(d->st_backlog);
 
@@ -1259,8 +1295,9 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_busy, RRDSET_FLAG_DETAIL);
 
-                d->rd_busy_busy =
-                    rrddim_add(d->st_busy, "busy", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+                d->rd_busy_busy = rrddim_add(d->st_busy, "busy", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_busy);
             }
             else rrdset_next(d->st_busy);
 
@@ -1288,6 +1325,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                 rrdset_flag_set(d->st_util, RRDSET_FLAG_DETAIL);
 
                 d->rd_util_utilization = rrddim_add(d->st_util, "utilization", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                add_labels_to_disk(d, d->st_util);
             }
             else rrdset_next(d->st_util);
 
@@ -1326,6 +1365,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 d->rd_mops_reads  = rrddim_add(d->st_mops, "reads",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_mops_writes = rrddim_add(d->st_mops, "writes", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_mops);
             }
             else rrdset_next(d->st_mops);
 
@@ -1358,7 +1399,10 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                 rrdset_flag_set(d->st_ext_mops, RRDSET_FLAG_DETAIL);
 
                 d->rd_mops_discards = rrddim_add(d->st_ext_mops, "discards", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-            } else
+
+                add_labels_to_disk(d, d->st_ext_mops);
+            }
+            else
                 rrdset_next(d->st_ext_mops);
 
             rrddim_set_by_pointer(d->st_ext_mops, d->rd_mops_discards, mdiscards);
@@ -1391,6 +1435,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 d->rd_iotime_reads  = rrddim_add(d->st_iotime, "reads",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_iotime_writes = rrddim_add(d->st_iotime, "writes", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_iotime);
             }
             else rrdset_next(d->st_iotime);
 
@@ -1422,9 +1468,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                 d->rd_iotime_discards = rrddim_add(d->st_ext_iotime, "discards", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                 if (do_fl_stats)
-                    d->rd_iotime_flushes =
-                        rrddim_add(d->st_ext_iotime, "flushes", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-            } else
+                    d->rd_iotime_flushes = rrddim_add(d->st_ext_iotime, "flushes", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                add_labels_to_disk(d, d->st_ext_iotime);
+            }
+            else
                 rrdset_next(d->st_ext_iotime);
 
             last_discardms = rrddim_set_by_pointer(d->st_ext_iotime, d->rd_iotime_discards, discardms);
@@ -1465,6 +1513,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_await_reads  = rrddim_add(d->st_await, "reads",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
                     d->rd_await_writes = rrddim_add(d->st_await, "writes", NULL, -1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_await);
                 }
                 else rrdset_next(d->st_await);
 
@@ -1494,18 +1544,22 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_await_discards = rrddim_add(d->st_ext_await, "discards", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
                     if (do_fl_stats)
-                        d->rd_await_flushes =
-                            rrddim_add(d->st_ext_await, "flushes", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
-                } else
+                        d->rd_await_flushes = rrddim_add(d->st_ext_await, "flushes", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_ext_await);
+                }
+                else
                     rrdset_next(d->st_ext_await);
 
                 rrddim_set_by_pointer(
                     d->st_ext_await, d->rd_await_discards,
                     (discards - last_discards) ? (discardms - last_discardms) / (discards - last_discards) : 0);
+
                 if (do_fl_stats)
                     rrddim_set_by_pointer(
                         d->st_ext_await, d->rd_await_flushes,
                         (flushes - last_flushes) ? (flushms - last_flushms) / (flushes - last_flushes) : 0);
+
                 rrdset_done(d->st_ext_await);
             }
 
@@ -1534,6 +1588,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_avgsz_reads  = rrddim_add(d->st_avgsz, "reads",  NULL, d->sector_size, 1024,      RRD_ALGORITHM_ABSOLUTE);
                     d->rd_avgsz_writes = rrddim_add(d->st_avgsz, "writes", NULL, d->sector_size * -1, 1024, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_avgsz);
                 }
                 else rrdset_next(d->st_avgsz);
 
@@ -1561,9 +1617,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     rrdset_flag_set(d->st_ext_avgsz, RRDSET_FLAG_DETAIL);
 
-                    d->rd_avgsz_discards =
-                        rrddim_add(d->st_ext_avgsz, "discards", NULL, d->sector_size, 1024, RRD_ALGORITHM_ABSOLUTE);
-                } else
+                    d->rd_avgsz_discards = rrddim_add(d->st_ext_avgsz, "discards", NULL, d->sector_size, 1024, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_ext_avgsz);
+                }
+                else
                     rrdset_next(d->st_ext_avgsz);
 
                 rrddim_set_by_pointer(
@@ -1599,8 +1657,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                     rrdset_flag_set(d->st_svctm, RRDSET_FLAG_DETAIL);
 
                     d->rd_svctm_svctm = rrddim_add(d->st_svctm, "svctm", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_svctm);
                 }
-                else rrdset_next(d->st_svctm);
+                else
+                    rrdset_next(d->st_svctm);
 
                 rrddim_set_by_pointer(d->st_svctm, d->rd_svctm_svctm, ((reads - last_reads) + (writes - last_writes)) ? (busy_ms - last_busy_ms) / ((reads - last_reads) + (writes - last_writes)) : 0);
                 rrdset_done(d->st_svctm);
@@ -1705,8 +1766,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                     d->rd_bcache_hit_ratio_1hour = rrddim_add(d->st_bcache_hit_ratio, "1hour", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
                     d->rd_bcache_hit_ratio_1day  = rrddim_add(d->st_bcache_hit_ratio, "1day",  NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
                     d->rd_bcache_hit_ratio_total = rrddim_add(d->st_bcache_hit_ratio, "ever",  NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_bcache_hit_ratio);
                 }
-                else rrdset_next(d->st_bcache_hit_ratio);
+                else
+                    rrdset_next(d->st_bcache_hit_ratio);
 
                 rrddim_set_by_pointer(d->st_bcache_hit_ratio, d->rd_bcache_hit_ratio_5min, stats_five_minute_cache_hit_ratio);
                 rrddim_set_by_pointer(d->st_bcache_hit_ratio, d->rd_bcache_hit_ratio_1hour, stats_hour_cache_hit_ratio);
@@ -1735,8 +1799,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_bcache_rate_congested = rrddim_add(d->st_bcache_rates, "congested", NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
                     d->rd_bcache_rate_writeback = rrddim_add(d->st_bcache_rates, "writeback", NULL, -1, 1024, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_bcache_rates);
                 }
-                else rrdset_next(d->st_bcache_rates);
+                else
+                    rrdset_next(d->st_bcache_rates);
 
                 rrddim_set_by_pointer(d->st_bcache_rates, d->rd_bcache_rate_writeback, writeback_rate);
                 rrddim_set_by_pointer(d->st_bcache_rates, d->rd_bcache_rate_congested, cache_congested);
@@ -1761,8 +1828,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                     );
 
                     d->rd_bcache_dirty_size = rrddim_add(d->st_bcache_size, "dirty", NULL,  1, 1024 * 1024, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_bcache_size);
                 }
-                else rrdset_next(d->st_bcache_size);
+                else
+                    rrdset_next(d->st_bcache_size);
 
                 rrddim_set_by_pointer(d->st_bcache_size, d->rd_bcache_dirty_size, dirty_data);
                 rrdset_done(d->st_bcache_size);
@@ -1786,8 +1856,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                     );
 
                     d->rd_bcache_available_percent = rrddim_add(d->st_bcache_usage, "avail", NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
+
+                    add_labels_to_disk(d, d->st_bcache_usage);
                 }
-                else rrdset_next(d->st_bcache_usage);
+                else
+                    rrdset_next(d->st_bcache_usage);
 
                 rrddim_set_by_pointer(d->st_bcache_usage, d->rd_bcache_available_percent, cache_available_percent);
                 rrdset_done(d->st_bcache_usage);
@@ -1813,8 +1886,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_bcache_cache_read_races = rrddim_add(d->st_bcache_cache_read_races, "races",  NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     d->rd_bcache_cache_io_errors  = rrddim_add(d->st_bcache_cache_read_races, "errors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                    add_labels_to_disk(d, d->st_bcache_cache_read_races);
                 }
-                else rrdset_next(d->st_bcache_cache_read_races);
+                else
+                    rrdset_next(d->st_bcache_cache_read_races);
 
                 rrddim_set_by_pointer(d->st_bcache_cache_read_races, d->rd_bcache_cache_read_races, cache_read_races);
                 rrddim_set_by_pointer(d->st_bcache_cache_read_races, d->rd_bcache_cache_io_errors, cache_io_errors);
@@ -1849,8 +1925,11 @@ int do_proc_diskstats(int update_every, usec_t dt) {
                     d->rd_bcache_misses          = rrddim_add(d->st_bcache, "misses",     NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     d->rd_bcache_miss_collisions = rrddim_add(d->st_bcache, "collisions", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     d->rd_bcache_readaheads      = rrddim_add(d->st_bcache, "readaheads", NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                    add_labels_to_disk(d, d->st_bcache);
                 }
-                else rrdset_next(d->st_bcache);
+                else
+                    rrdset_next(d->st_bcache);
 
                 rrddim_set_by_pointer(d->st_bcache, d->rd_bcache_hits, stats_total_cache_hits);
                 rrddim_set_by_pointer(d->st_bcache, d->rd_bcache_misses, stats_total_cache_misses);
@@ -1884,6 +1963,8 @@ int do_proc_diskstats(int update_every, usec_t dt) {
 
                     d->rd_bcache_bypass_hits   = rrddim_add(d->st_bcache_bypass, "hits",   NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                     d->rd_bcache_bypass_misses = rrddim_add(d->st_bcache_bypass, "misses", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
+
+                    add_labels_to_disk(d, d->st_bcache_bypass);
                 }
                 else rrdset_next(d->st_bcache_bypass);
 
