@@ -210,7 +210,7 @@ int do_proc_interrupts(int update_every, usec_t dt) {
                 snprintfz(id, 50, "cpu%d_interrupts", c);
 
                 char title[100+1];
-                snprintfz(title, 100, "CPU%d Interrupts", c);
+                snprintfz(title, 100, "CPU Interrupts");
                 core_st[c] = rrdset_create_localhost(
                         "cpu"
                         , id
@@ -225,6 +225,10 @@ int do_proc_interrupts(int update_every, usec_t dt) {
                         , update_every
                         , RRDSET_TYPE_STACKED
                 );
+
+                char core[50+1];
+                snprintfz(core, 50, "cpu%d", c);
+                rrdlabels_add(core_st[c]->state->chart_labels, "cpu", core, RRDLABEL_SRC_AUTO);
             }
             else rrdset_next(core_st[c]);
 
