@@ -55,11 +55,18 @@ class RuntimeCounters:
             self.penalty = round(min(self.retries * self.update_every / 2, MAX_PENALTY))
 
 
+def clean_module_name(name):
+    if name.startswith('pythond_'):
+        return name[8:]
+    return name
+
+
 class SimpleService(PythonDLimitedLogger, object):
     """
     Prototype of Service class.
     Implemented basic functionality to run jobs by `python.d.plugin`
     """
+
     def __init__(self, configuration, name=''):
         """
         :param configuration: <dict>
@@ -70,7 +77,7 @@ class SimpleService(PythonDLimitedLogger, object):
         self.order = list()
         self.definitions = dict()
 
-        self.module_name = self.__module__
+        self.module_name = clean_module_name(self.__module__)
         self.job_name = configuration.pop('job_name')
         self.override_name = configuration.pop('override_name')
         self.fake_name = None
@@ -88,8 +95,9 @@ class SimpleService(PythonDLimitedLogger, object):
 
     @property
     def name(self):
-        if self.job_name and self.job_name != self.module_name:
-            return '_'.join([self.module_name, self.override_name or self.job_name])
+        name = self.override_name or self.job_name
+        if name and name != self.module_name:
+            return '_'.join([self.module_name, name])
         return self.module_name
 
     def actual_name(self):
