@@ -2,6 +2,14 @@
 
 #include "exporting_engine.h"
 
+
+bool exporting_labels_filter_callback(const char *name, const char *value, RRDLABEL_SRC ls, void *data) {
+    (void)name;
+    (void)value;
+    struct instance *instance = (struct instance *)data;
+    return should_send_label(instance, ls);
+}
+
 /**
  * Check if the connector instance should export the host metrics
  *
@@ -16,19 +24,19 @@ int rrdhost_is_exportable(struct instance *instance, RRDHOST *host)
 
     RRDHOST_FLAGS *flags = &host->exporting_flags[instance->index];
 
-    if (unlikely((*flags & (RRDHOST_FLAG_BACKEND_SEND | RRDHOST_FLAG_BACKEND_DONT_SEND)) == 0)) {
+    if (unlikely((*flags & (RRDHOST_FLAG_EXPORTING_SEND | RRDHOST_FLAG_EXPORTING_DONT_SEND)) == 0)) {
         char *host_name = (host == localhost) ? "localhost" : host->hostname;
 
         if (!instance->config.hosts_pattern || simple_pattern_matches(instance->config.hosts_pattern, host_name)) {
-            *flags |= RRDHOST_FLAG_BACKEND_SEND;
+            *flags |= RRDHOST_FLAG_EXPORTING_SEND;
             info("enabled exporting of host '%s' for instance '%s'", host_name, instance->config.name);
         } else {
-            *flags |= RRDHOST_FLAG_BACKEND_DONT_SEND;
+            *flags |= RRDHOST_FLAG_EXPORTING_DONT_SEND;
             info("disabled exporting of host '%s' for instance '%s'", host_name, instance->config.name);
         }
     }
 
-    if (likely(*flags & RRDHOST_FLAG_BACKEND_SEND))
+    if (likely(*flags & RRDHOST_FLAG_EXPORTING_SEND))
         return 1;
     else
         return 0;
@@ -47,6 +55,10 @@ int rrdset_is_exportable(struct instance *instance, RRDSET *st)
     RRDHOST *host = st->rrdhost;
 #endif
 
+    // Do not export anomaly rates charts.
+    if (st->state && st->state->is_ar_chart)
+        return 0;
+
     if (st->exporting_flags == NULL)
         st->exporting_flags = callocz(instance->engine->instance_num, sizeof(size_t));
 
@@ -61,18 +73,18 @@ int rrdset_is_exportable(struct instance *instance, RRDSET *st)
             *flags |= RRDSET_FLAG_EXPORTING_SEND;
         else {
             *flags |= RRDSET_FLAG_EXPORTING_IGNORE;
-            debug(D_BACKEND, "BACKEND: not sending chart '%s' of host '%s', because it is disabled for backends.", st->id, host->hostname);
+            debug(D_EXPORTING, "EXPORTING: not sending chart '%s' of host '%s', because it is disabled for exporting.", st->id, host->hostname);
             return 0;
         }
     }
 
     if(unlikely(!rrdset_is_available_for_exporting_and_alarms(st))) {
-        debug(D_BACKEND, "BACKEND: not sending chart '%s' of host '%s', because it is not available for backends.", st->id, host->hostname);
+        debug(D_EXPORTING, "EXPORTING: not sending chart '%s' of host '%s', because it is not available for exporting.", st->id, host->hostname);
         return 0;
     }
 
     if(unlikely(st->rrd_memory_mode == RRD_MEMORY_MODE_NONE && !(EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AS_COLLECTED))) {
-        debug(D_BACKEND, "BACKEND: not sending chart '%s' of host '%s' because its memory mode is '%s' and the backend requires database access.", st->id, host->hostname, rrd_memory_mode_name(host->rrd_memory_mode));
+        debug(D_EXPORTING, "EXPORTING: not sending chart '%s' of host '%s' because its memory mode is '%s' and the exporting engine requires database access.", st->id, host->hostname, rrd_memory_mode_name(host->rrd_memory_mode));
         return 0;
     }
 
