@@ -15,10 +15,13 @@ public:
     unsigned MinTrainSamples;
     unsigned TrainEvery;
 
+    unsigned DBEngineAnomalyRateEvery;
+
     unsigned DiffN;
     unsigned SmoothN;
     unsigned LagN;
 
+    double RandomSamplingRatio;
     unsigned MaxKMeansIters;
 
     double DimensionAnomalyScoreThreshold;
@@ -30,6 +33,8 @@ public:
     double ADWindowRateThreshold;
     double ADDimensionRateThreshold;
 
+    bool StreamADCharts;
+
     std::string HostsToSkip;
     SIMPLE_PATTERN *SP_HostsToSkip;
 
@@ -37,6 +42,7 @@ public:
     SIMPLE_PATTERN *SP_ChartsToSkip;
 
     std::string AnomalyDBPath;
+    std::vector<uint32_t> RandomNums;
 
     void readMLConfig();
 };
