@@ -4,6 +4,15 @@
 
 #if !defined(ENABLE_ML)
 
+bool ml_capable() {
+    return false;
+}
+
+bool ml_enabled(RRDHOST *RH) {
+    (void) RH;
+    return false;
+}
+
 void ml_init(void) {}
 
 void ml_new_host(RRDHOST *RH) { (void) RH; }
@@ -38,4 +47,19 @@ char *ml_get_anomaly_event_info(RRDHOST *RH, const char *AnomalyDetectorName,
     return NULL;
 }
 
+void ml_process_rrdr(RRDR *R, int MaxAnomalyRates) {
+    (void) R;
+    (void) MaxAnomalyRates;
+}
+
+void ml_dimension_update_name(RRDSET *RS, RRDDIM *RD, const char *name) {
+    (void) RS;
+    (void) RD;
+    (void) name;
+}
+
+bool ml_streaming_enabled() {
+    return false;
+}
+
 #endif
