@@ -17,6 +17,7 @@ package_tree="
   autoconf
   autoconf-archive
   autogen
+  libatomic1
   libtool
   pkg-config
   tar
@@ -30,7 +31,6 @@ package_tree="
   libuv1-dev
   liblz4-dev
   libssl-dev
-  libjudy-dev
   libelf-dev
   python3
   "
