@@ -2,9 +2,43 @@
 
 #include "json_wrapper.h"
 
+struct value_output {
+    int c;
+    BUFFER *wb;
+};
+
+static int value_list_output(const char *name, void *entry, void *data) {
+    (void)name;
+
+    struct value_output *ap = (struct value_output *)data;
+    BUFFER *wb = ap->wb;
+    char *output = (char *) entry;
+    if(ap->c) buffer_strcat(wb, ",");
+    buffer_strcat(wb, output);
+    (ap->c)++;
+    return 0;
+}
+
+static int fill_formatted_callback(const char *name, const char *value, RRDLABEL_SRC ls, void *data) {
+    (void)ls;
+    DICTIONARY *dict = (DICTIONARY *)data;
+    char n[RRD_ID_LENGTH_MAX * 2 + 2];
+    char output[RRD_ID_LENGTH_MAX * 2 + 8];
+    char v[RRD_ID_LENGTH_MAX * 2 + 1];
+
+    sanitize_json_string(v, (char *)value, RRD_ID_LENGTH_MAX * 2);
+    int len = snprintfz(output, RRD_ID_LENGTH_MAX * 2 + 7, "[\"%s\", \"%s\"]", name, v);
+    snprintfz(n, RRD_ID_LENGTH_MAX * 2, "%s:%s", name, v);
+    dictionary_set(dict, n, output, len + 1);
+
+    return 1;
+}
+
 void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS options, int string_value,
-                             struct context_param *context_param_list, char *chart_label_key)
+    RRDR_GROUPING group_method, QUERY_PARAMS *rrdset_query_data)
 {
+    struct context_param *context_param_list = rrdset_query_data->context_param_list;
+    char *chart_label_key = rrdset_query_data->chart_label_key;
 
     RRDDIM *temp_rd = context_param_list ? context_param_list->rd : NULL;
     int should_lock = (!context_param_list || !(context_param_list->flags & CONTEXT_FLAGS_ARCHIVE));
@@ -42,7 +76,8 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
                        "   %slast_entry%s: %u,\n"
                        "   %sbefore%s: %u,\n"
                        "   %safter%s: %u,\n"
-                       "   %sdimension_names%s: ["
+                       "   %sgroup%s: %s%s%s,\n"
+                       "   %soptions%s: %s"
                    , kq, kq
                    , kq, kq, sq, context_mode && temp_rd?r->st->context:r->st->id, sq
                    , kq, kq, sq, context_mode && temp_rd?r->st->context:r->st->name, sq
@@ -52,7 +87,13 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
                    , kq, kq, (uint32_t) (context_param_list ? context_param_list->last_entry_t : rrdset_last_entry_t_nolock(r->st))
                    , kq, kq, (uint32_t)r->before
                    , kq, kq, (uint32_t)r->after
-                   , kq, kq);
+                   , kq, kq, sq, web_client_api_request_v1_data_group_to_string(group_method), sq
+                   , kq, kq, sq);
+
+    web_client_api_request_v1_data_options_to_string(wb, r->internal.query_options);
+
+    buffer_sprintf(wb, "%s,\n   %sdimension_names%s: [", sq, kq, kq);
+
     if (should_lock)
         rrdset_unlock(r->st);
 
@@ -98,6 +139,49 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
     }
     buffer_strcat(wb, "],\n");
 
+    if (rrdset_query_data->show_dimensions) {
+        buffer_sprintf(wb, "   %sfull_dimension_list%s: [", kq, kq);
+
+        char name[RRD_ID_LENGTH_MAX * 2 + 2];
+        char output[RRD_ID_LENGTH_MAX * 2 + 8];
+
+        struct value_output co = {.c = 0, .wb = wb};
+
+        DICTIONARY *dict = dictionary_create(DICTIONARY_FLAG_SINGLE_THREADED);
+        for (i = 0, rd = temp_rd ? temp_rd : r->st->dimensions; rd; rd = rd->next) {
+            snprintfz(name, RRD_ID_LENGTH_MAX * 2, "%s:%s", rd->id, rd->name);
+            int len = snprintfz(output, RRD_ID_LENGTH_MAX * 2 + 7, "[\"%s\",\"%s\"]", rd->id, rd->name);
+            dictionary_set(dict, name, output, len+1);
+        }
+        dictionary_walkthrough_read(dict, value_list_output, &co);
+        dictionary_destroy(dict);
+
+        co.c = 0;
+        buffer_sprintf(wb, "],\n   %sfull_chart_list%s: [", kq, kq);
+        dict = dictionary_create(DICTIONARY_FLAG_SINGLE_THREADED);
+        for (i = 0, rd = temp_rd ? temp_rd : r->st->dimensions; rd; rd = rd->next) {
+            int len = snprintfz(output, RRD_ID_LENGTH_MAX * 2 + 7, "[\"%s\",\"%s\"]", rd->rrdset->id, rd->rrdset->name);
+            snprintfz(name, RRD_ID_LENGTH_MAX * 2, "%s:%s", rd->rrdset->id, rd->rrdset->name);
+            dictionary_set(dict, name, output, len + 1);
+        }
+
+        dictionary_walkthrough_read(dict, value_list_output, &co);
+        dictionary_destroy(dict);
+
+        RRDSET *st;
+        co.c = 0;
+        buffer_sprintf(wb, "],\n   %sfull_chart_labels%s: [", kq, kq);
+        dict = dictionary_create(DICTIONARY_FLAG_SINGLE_THREADED);
+        for (i = 0, rd = temp_rd ? temp_rd : r->st->dimensions; rd; rd = rd->next) {
+            st = rd->rrdset;
+            if (st->state && st->state->chart_labels)
+                rrdlabels_walkthrough_read(st->state->chart_labels, fill_formatted_callback, dict);
+        }
+        dictionary_walkthrough_read(dict, value_list_output, &co);
+        dictionary_destroy(dict);
+        buffer_strcat(wb, "],\n");
+    }
+
     // Composite charts
     if (context_mode && temp_rd) {
         buffer_sprintf(
@@ -133,8 +217,6 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
             char *label_key = NULL;
             int keys = 0;
             while (pattern && (label_key = simple_pattern_iterate(&pattern))) {
-                uint32_t key_hash = simple_hash(label_key);
-                struct label *current_label;
 
                 if (keys)
                     buffer_strcat(wb, ", ");
@@ -149,13 +231,7 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
                     if (i)
                         buffer_strcat(wb, ", ");
 
-                    current_label = rrdset_lookup_label_key(rd->rrdset, label_key, key_hash);
-                    if (current_label) {
-                        buffer_strcat(wb, sq);
-                        buffer_strcat(wb, current_label->value);
-                        buffer_strcat(wb, sq);
-                    } else
-                        buffer_strcat(wb, "null");
+                    rrdlabels_get_value_to_buffer_or_null(rd->rrdset->state->chart_labels, wb, label_key, sq, "null");
                     i++;
                 }
                 if (!i) {
@@ -181,7 +257,7 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
         if(i) buffer_strcat(wb, ", ");
         i++;
 
-        calculated_number value = rd->last_stored_value;
+        NETDATA_DOUBLE value = rd->last_stored_value;
         if (NAN == value)
             buffer_strcat(wb, "null");
         else
@@ -206,13 +282,13 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
 
     i = 0;
     if(rows) {
-        calculated_number total = 1;
+        NETDATA_DOUBLE total = 1;
 
         if(unlikely(options & RRDR_OPTION_PERCENTAGE)) {
             total = 0;
             for(c = 0, rd = temp_rd?temp_rd:r->st->dimensions; rd && c < r->d ;c++, rd = rd->next) {
-                calculated_number *cn = &r->v[ (rrdr_rows(r) - 1) * r->d ];
-                calculated_number n = cn[c];
+                NETDATA_DOUBLE *cn = &r->v[ (rrdr_rows(r) - 1) * r->d ];
+                NETDATA_DOUBLE n = cn[c];
 
                 if(likely((options & RRDR_OPTION_ABSOLUTE) && n < 0))
                     n = -n;
@@ -230,9 +306,9 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
             if(i) buffer_strcat(wb, ", ");
             i++;
 
-            calculated_number *cn = &r->v[ (rrdr_rows(r) - 1) * r->d ];
+            NETDATA_DOUBLE *cn = &r->v[ (rrdr_rows(r) - 1) * r->d ];
             RRDR_VALUE_FLAGS *co = &r->o[ (rrdr_rows(r) - 1) * r->d ];
-            calculated_number n = cn[c];
+            NETDATA_DOUBLE n = cn[c];
 
             if(co[c] & RRDR_VALUE_EMPTY) {
                 if(options & RRDR_OPTION_NULL2ZERO)
@@ -267,12 +343,21 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
 
     rrdr_buffer_print_format(wb, format);
 
+    buffer_sprintf(wb, "%s,\n"
+                       "   %sdb_points_per_tier%s: [ "
+                   , sq
+                   , kq, kq
+                   );
+
+    for(int tier = 0; tier < storage_tiers ; tier++)
+        buffer_sprintf(wb, "%s%zu", tier>0?", ":"", r->internal.tier_points_read[tier]);
+
+    buffer_strcat(wb, " ]");
+
     if((options & RRDR_OPTION_CUSTOM_VARS) && (options & RRDR_OPTION_JSON_WRAP)) {
-        buffer_sprintf(wb, "%s,\n   %schart_variables%s: ", sq, kq, kq);
+        buffer_sprintf(wb, ",\n   %schart_variables%s: ", kq, kq);
         health_api_v1_chart_custom_variables2json(r->st, wb);
     }
-    else
-        buffer_sprintf(wb, "%s", sq);
 
     buffer_sprintf(wb, ",\n   %sresult%s: ", kq, kq);
 
@@ -280,6 +365,27 @@ void rrdr_json_wrapper_begin(RRDR *r, BUFFER *wb, uint32_t format, RRDR_OPTIONS
     //info("JSONWRAPPER(): %s: END", r->st->id);
 }
 
+void rrdr_json_wrapper_anomaly_rates(RRDR *r, BUFFER *wb, uint32_t format, uint32_t options, int string_value) {
+    (void)r;
+    (void)format;
+
+    char kq[2] = "",                    // key quote
+        sq[2] = "";                     // string quote
+
+    if( options & RRDR_OPTION_GOOGLE_JSON ) {
+        kq[0] = '\0';
+        sq[0] = '\'';
+    }
+    else {
+        kq[0] = '"';
+        sq[0] = '"';
+    }
+
+    if(string_value) buffer_strcat(wb, sq);
+
+    buffer_sprintf(wb, ",\n   %sanomaly_rates%s: ", kq, kq);
+}
+
 void rrdr_json_wrapper_end(RRDR *r, BUFFER *wb, uint32_t format, uint32_t options, int string_value) {
     (void)format;
 
