@@ -8,10 +8,10 @@
 
 extern void grouping_init_des(void);
 
-extern void *grouping_create_des(RRDR *r);
+extern void grouping_create_des(RRDR *r, const char *options __maybe_unused);
 extern void grouping_reset_des(RRDR *r);
 extern void grouping_free_des(RRDR *r);
-extern void grouping_add_des(RRDR *r, calculated_number value);
-extern calculated_number grouping_flush_des(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr);
+extern void grouping_add_des(RRDR *r, NETDATA_DOUBLE value);
+extern NETDATA_DOUBLE grouping_flush_des(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr);
 
 #endif //NETDATA_API_QUERIES_DES_H
